@@ -38,17 +38,20 @@ to `:py:class`pyfakefs.fake_filesystem_unittest.TestCase`.
 import doctest
 import functools
 import inspect
+import linecache
 import shutil
 import sys
 import tempfile
+import tokenize
 import unittest
 import warnings
 
 from pyfakefs.deprecator import Deprecator
-from pyfakefs.fake_filesystem import set_uid, set_gid, reset_ids
+from pyfakefs.fake_filesystem import set_uid, set_gid, reset_ids, PatchMode
 from pyfakefs.helpers import IS_PYPY
-from pyfakefs.patched_packages import get_modules_to_patch, \
-    get_classes_to_patch, get_fake_module_classes
+from pyfakefs.patched_packages import (
+    get_modules_to_patch, get_classes_to_patch, get_fake_module_classes
+)
 
 try:
     from importlib.machinery import ModuleSpec
@@ -69,7 +72,6 @@ if use_scandir:
 
 OS_MODULE = 'nt' if sys.platform == 'win32' else 'posix'
 PATH_MODULE = 'ntpath' if sys.platform == 'win32' else 'posixpath'
-BUILTIN_MODULE = '__builtin__'
 
 
 def patchfs(_func=None, *,
@@ -77,7 +79,8 @@ def patchfs(_func=None, *,
             modules_to_reload=None,
             modules_to_patch=None,
             allow_root_user=True,
-            use_known_patches=True):
+            use_known_patches=True,
+            patch_open_code=PatchMode.OFF):
     """Convenience decorator to use patcher with additional parameters in a
     test function.
 
@@ -100,7 +103,8 @@ def patchfs(_func=None, *,
                     modules_to_reload=modules_to_reload,
                     modules_to_patch=modules_to_patch,
                     allow_root_user=allow_root_user,
-                    use_known_patches=use_known_patches) as p:
+                    use_known_patches=use_known_patches,
+                    patch_open_code=patch_open_code) as p:
                 kwargs['fs'] = p.fs
                 return f(*args, **kwargs)
 
@@ -117,12 +121,14 @@ def patchfs(_func=None, *,
     return wrap_patchfs
 
 
-def load_doctests(loader, tests, ignore, module,
-                  additional_skip_names=None,
-                  modules_to_reload=None,
-                  modules_to_patch=None,
-                  allow_root_user=True,
-                  use_known_patches=True):  # pylint: disable=unused-argument
+def load_doctests(
+        loader, tests, ignore, module,
+        additional_skip_names=None,
+        modules_to_reload=None,
+        modules_to_patch=None,
+        allow_root_user=True,
+        use_known_patches=True,
+        patch_open_code=PatchMode.OFF):  # pylint: disable=unused-argument
     """Load the doctest tests for the specified module into unittest.
         Args:
             loader, tests, ignore : arguments passed in from `load_tests()`
@@ -135,7 +141,8 @@ def load_doctests(loader, tests, ignore, module,
                        modules_to_reload=modules_to_reload,
                        modules_to_patch=modules_to_patch,
                        allow_root_user=allow_root_user,
-                       use_known_patches=use_known_patches)
+                       use_known_patches=use_known_patches,
+                       patch_open_code=patch_open_code)
     globs = _patcher.replace_globs(vars(module))
     tests.addTests(doctest.DocTestSuite(module,
                                         globs=globs,
@@ -161,8 +168,6 @@ class TestCaseMixin:
         modules_to_patch: A dictionary of fake modules mapped to the
             fully qualified patched module names. Can be used to add patching
             of modules not provided by `pyfakefs`.
-        use_known_patches: If True (the default), some patches for commonly
-            used packges are applied which make them usable with pyfakes.
 
     If you specify some of these attributes here and you have DocTests,
     consider also specifying the same arguments to :py:func:`load_doctests`.
@@ -199,7 +204,8 @@ class TestCaseMixin:
                       modules_to_reload=None,
                       modules_to_patch=None,
                       allow_root_user=True,
-                      use_known_patches=True):
+                      use_known_patches=True,
+                      patch_open_code=PatchMode.OFF):
         """Bind the file-related modules to the :py:class:`pyfakefs` fake file
         system instead of the real file system.  Also bind the fake `open()`
         function.
@@ -222,7 +228,8 @@ class TestCaseMixin:
             modules_to_reload=modules_to_reload,
             modules_to_patch=modules_to_patch,
             allow_root_user=allow_root_user,
-            use_known_patches=use_known_patches
+            use_known_patches=use_known_patches,
+            patch_open_code=patch_open_code
         )
 
         self._stubber.setUp()
@@ -256,9 +263,7 @@ class TestCase(unittest.TestCase, TestCaseMixin):
     def __init__(self, methodName='runTest',
                  additional_skip_names=None,
                  modules_to_reload=None,
-                 modules_to_patch=None,
-                 allow_root_user=True,
-                 use_known_patches=True):
+                 modules_to_patch=None):
         """Creates the test class instance and the patcher used to stub out
         file system related modules.
 
@@ -271,8 +276,6 @@ class TestCase(unittest.TestCase, TestCaseMixin):
         self.additional_skip_names = additional_skip_names
         self.modules_to_reload = modules_to_reload
         self.modules_to_patch = modules_to_patch
-        self.allow_root_user = allow_root_user
-        self.use_known_patches = use_known_patches
 
     @Deprecator('add_real_file')
     def copyRealFile(self, real_file_path, fake_file_path=None,
@@ -338,8 +341,26 @@ class Patcher:
     '''Stub nothing that is imported within these modules.
     `sys` is included to prevent `sys.path` from being stubbed with the fake
     `os.path`.
+    The `pytest` and `py` modules are used by pytest and have to access the
+    real file system.
+    The `linecache` module is used to read the test file in case of test
+    failure to get traceback information before test tear down.
+    In order to make sure that reading the test file is not faked,
+    we skip faking the module.
+    We also have to set back the cached open function in tokenize.
     '''
-    SKIPMODULES = {None, fake_filesystem, fake_filesystem_shutil, sys}
+    SKIPMODULES = {
+        None, fake_filesystem, fake_filesystem_shutil,
+        sys, linecache, tokenize
+    }
+    try:
+        import py
+        import pytest
+        SKIPMODULES.add(py)
+        SKIPMODULES.add(pytest)
+    except ImportError:
+        pass
+
     assert None in SKIPMODULES, ("sys.modules contains 'None' values;"
                                  " must skip them.")
 
@@ -349,25 +370,51 @@ class Patcher:
 
     def __init__(self, additional_skip_names=None,
                  modules_to_reload=None, modules_to_patch=None,
-                 allow_root_user=True, use_known_patches=True):
-        """For a description of the arguments, see TestCase.__init__"""
+                 allow_root_user=True, use_known_patches=True,
+                 patch_open_code=PatchMode.OFF):
+        """
+        Args:
+            additional_skip_names: names of modules inside of which no module
+                replacement shall be performed, in addition to the names in
+                :py:attr:`fake_filesystem_unittest.Patcher.SKIPNAMES`.
+                Instead of the module names, the modules themselves
+                may be used.
+            modules_to_reload: A list of modules that need to be reloaded
+                to be patched dynamically; may be needed if the module
+                imports file system modules under an alias
+
+                .. caution:: Reloading modules may have unwanted side effects.
+            modules_to_patch: A dictionary of fake modules mapped to the
+                fully qualified patched module names. Can be used to add
+                patching of modules not provided by `pyfakefs`.
+            allow_root_user: If True (default), if the test is run as root
+                user, the user in the fake file system is also considered a
+                root user, otherwise it is always considered a regular user.
+            use_known_patches: If True (the default), some patches for commonly
+                used packages are applied which make them usable with pyfakefs.
+            patch_open_code: If True, `io.open_code` is patched. The default
+                is not to patch it, as it mostly is used to load compiled
+                modules that are not in the fake file system.
+        """
 
         if not allow_root_user:
             # set non-root IDs even if the real user is root
             set_uid(1)
             set_gid(1)
 
-        self._skipNames = self.SKIPNAMES.copy()
+        self._skip_names = self.SKIPNAMES.copy()
         # save the original open function for use in pytest plugin
         self.original_open = open
         self.fake_open = None
+        self.patch_open_code = patch_open_code
 
         if additional_skip_names is not None:
             skip_names = [m.__name__ if inspect.ismodule(m) else m
                           for m in additional_skip_names]
-            self._skipNames.update(skip_names)
+            self._skip_names.update(skip_names)
 
         self._fake_module_classes = {}
+        self._unfaked_module_classes = {}
         self._class_modules = {}
         self._init_fake_module_classes()
 
@@ -388,12 +435,14 @@ class Patcher:
 
         # Attributes set by _refresh()
         self._modules = {}
+        self._skipped_modules = {}
         self._fct_modules = {}
         self._def_functions = []
         self._open_functions = {}
         self._stubs = None
         self.fs = None
         self.fake_modules = {}
+        self.unfaked_modules = {}
         self._dyn_patcher = None
 
         # _isStale is set by tearDown(), reset by _refresh()
@@ -422,12 +471,18 @@ class Patcher:
             self._fake_module_classes[
                 'pathlib'] = fake_pathlib.FakePathlibModule
             self._class_modules['Path'].append('pathlib')
+            self._unfaked_module_classes[
+                'pathlib'] = fake_pathlib.RealPathlibModule
         if pathlib2:
             self._fake_module_classes[
                 'pathlib2'] = fake_pathlib.FakePathlibModule
             self._class_modules['Path'].append('pathlib2')
+            self._unfaked_module_classes[
+                'pathlib2'] = fake_pathlib.RealPathlibModule
         self._fake_module_classes[
             'Path'] = fake_pathlib.FakePathlibPathModule
+        self._unfaked_module_classes[
+            'Path'] = fake_pathlib.RealPathlibPathModule
         if use_scandir:
             self._fake_module_classes[
                 'scandir'] = fake_scandir.FakeScanDirModule
@@ -474,9 +529,20 @@ class Patcher:
                     mod.__name__ in module_names
                     or inspect.isclass(mod) and
                     mod.__module__ in self._class_modules.get(name, []))
-        except AttributeError:
+        except Exception:
             # handle cases where the module has no __name__ or __module__
-            # attribute - see #460
+            # attribute - see #460, and any other exception triggered
+            # by inspect functions
+            return False
+
+    def _is_skipped_fs_module(self, mod, name, module_names):
+        try:
+            return (inspect.ismodule(mod) and
+                    mod.__name__ in module_names)
+        except Exception:
+            # handle cases where the module has no __name__ or __module__
+            # attribute - see #460, and any other exception triggered
+            # by inspect functions
             return False
 
     def _is_fs_function(self, fct):
@@ -486,35 +552,42 @@ class Patcher:
                     fct.__name__ in self._fake_module_functions and
                     fct.__module__ in self._fake_module_functions[
                         fct.__name__])
-        except AttributeError:
+        except Exception:
             # handle cases where the function has no __name__ or __module__
-            # attribute
+            # attribute, or any other exception in inspect functions
             return False
 
     def _def_values(self, item):
         """Find default arguments that are file-system functions to be
         patched in top-level functions and members of top-level classes."""
         # check for module-level functions
-        if inspect.isfunction(item):
-            if item.__defaults__:
-                for i, d in enumerate(item.__defaults__):
-                    if self._is_fs_function(d):
-                        yield item, i, d
-        elif inspect.isclass(item):
-            # check for methods in class (nested classes are ignored for now)
-            try:
-                for m in inspect.getmembers(item,
-                                            predicate=inspect.isfunction):
-                    m = m[1]
-                    if m.__defaults__:
-                        for i, d in enumerate(m.__defaults__):
-                            if self._is_fs_function(d):
-                                yield m, i, d
-            except Exception:
-                # Ignore any exception, examples:
-                # ImportError: No module named '_gdbm'
-                # _DontDoThat() (see #523)
-                pass
+        try:
+            if inspect.isfunction(item):
+                if item.__defaults__:
+                    for i, d in enumerate(item.__defaults__):
+                        if self._is_fs_function(d):
+                            yield item, i, d
+            elif inspect.isclass(item):
+                # check for methods in class
+                # (nested classes are ignored for now)
+                with warnings.catch_warnings():
+                    # ignore deprecation warnings, see #542
+                    warnings.filterwarnings(
+                        'ignore',
+                        category=DeprecationWarning
+                    )
+                    for m in inspect.getmembers(item,
+                                                predicate=inspect.isfunction):
+                        m = m[1]
+                        if m.__defaults__:
+                            for i, d in enumerate(m.__defaults__):
+                                if self._is_fs_function(d):
+                                    yield m, i, d
+        except Exception:
+            # Ignore any exception, examples:
+            # ImportError: No module named '_gdbm'
+            # _DontDoThat() (see #523)
+            pass
 
     def _find_modules(self):
         """Find and cache all modules that import file system modules.
@@ -525,15 +598,16 @@ class Patcher:
         module_names = list(self._fake_module_classes.keys()) + [PATH_MODULE]
         for name, module in list(sys.modules.items()):
             try:
-                if (module in self.SKIPMODULES or
-                        not inspect.ismodule(module) or
-                        module.__name__.split('.')[0] in self._skipNames):
+                if module in self.SKIPMODULES or not inspect.ismodule(module):
                     continue
-            except AttributeError:
+            except Exception:
                 # workaround for some py (part of pytest) versions
                 # where py.error has no __name__ attribute
                 # see https://github.com/pytest-dev/py/issues/73
+                # and any other exception triggered by inspect.ismodule
                 continue
+            skipped = (any([sn.startswith(module.__name__)
+                            for sn in self._skip_names]))
             module_items = module.__dict__.copy().items()
 
             # suppress specific pytest warning - see #466
@@ -547,9 +621,15 @@ class Patcher:
                 modules = {name: mod for name, mod in module_items
                            if self._is_fs_module(mod, name, module_names)}
 
+            if skipped:
+                for name, mod in modules.items():
+                    self._skipped_modules.setdefault(name, set()).add(
+                        (module, mod.__name__))
+                continue
+
             for name, mod in modules.items():
-                self._modules.setdefault(name, set()).add((module,
-                                                           mod.__name__))
+                self._modules.setdefault(name, set()).add(
+                    (module, mod.__name__))
             functions = {name: fct for name, fct in
                          module_items
                          if self._is_fs_function(fct)}
@@ -570,10 +650,15 @@ class Patcher:
         self._stubs = mox3_stubout.StubOutForTesting()
 
         self.fs = fake_filesystem.FakeFilesystem(patcher=self)
+        self.fs.patch_open_code = self.patch_open_code
         for name in self._fake_module_classes:
             self.fake_modules[name] = self._fake_module_classes[name](self.fs)
+            if hasattr(self.fake_modules[name], 'skip_names'):
+                self.fake_modules[name].skip_names = self._skip_names
         self.fake_modules[PATH_MODULE] = self.fake_modules['os'].path
         self.fake_open = fake_filesystem.FakeFileOpen(self.fs)
+        for name in self._unfaked_module_classes:
+            self.unfaked_modules[name] = self._unfaked_module_classes[name]()
 
         self._isStale = False
 
@@ -595,6 +680,8 @@ class Patcher:
             doctester.globs = self.replace_globs(doctester.globs)
 
         self.start_patching()
+        linecache.open = self.original_open
+        tokenize._builtin_open = self.original_open
 
         # the temp directory is assumed to exist at least in `tempfile1`,
         # so we create it here for convenience
@@ -604,29 +691,9 @@ class Patcher:
         if not self._patching:
             self._patching = True
 
-            for name, modules in self._modules.items():
-                for module, attr in modules:
-                    self._stubs.smart_set(
-                        module, name, self.fake_modules[attr])
-            for (name, ft_name, ft_mod), modules in self._fct_modules.items():
-                method, mod_name = self._fake_module_functions[ft_name][ft_mod]
-                fake_module = self.fake_modules[mod_name]
-                attr = method.__get__(fake_module, fake_module.__class__)
-                for module in modules:
-                    self._stubs.smart_set(module, name, attr)
-
-            for (fct, idx, ft) in self._def_functions:
-                method, mod_name = self._fake_module_functions[
-                    ft.__name__][ft.__module__]
-                fake_module = self.fake_modules[mod_name]
-                attr = method.__get__(fake_module, fake_module.__class__)
-                new_defaults = []
-                for i, d in enumerate(fct.__defaults__):
-                    if i == idx:
-                        new_defaults.append(attr)
-                    else:
-                        new_defaults.append(d)
-                fct.__defaults__ = tuple(new_defaults)
+            self.patch_modules()
+            self.patch_functions()
+            self.patch_defaults()
 
             self._dyn_patcher = DynamicPatcher(self)
             sys.meta_path.insert(0, self._dyn_patcher)
@@ -634,6 +701,39 @@ class Patcher:
                 if module.__name__ in sys.modules:
                     reload(module)
 
+    def patch_functions(self):
+        for (name, ft_name, ft_mod), modules in self._fct_modules.items():
+            method, mod_name = self._fake_module_functions[ft_name][ft_mod]
+            fake_module = self.fake_modules[mod_name]
+            attr = method.__get__(fake_module, fake_module.__class__)
+            for module in modules:
+                self._stubs.smart_set(module, name, attr)
+
+    def patch_modules(self):
+        for name, modules in self._modules.items():
+            for module, attr in modules:
+                self._stubs.smart_set(
+                    module, name, self.fake_modules[attr])
+        for name, modules in self._skipped_modules.items():
+            for module, attr in modules:
+                if attr in self.unfaked_modules:
+                    self._stubs.smart_set(
+                        module, name, self.unfaked_modules[attr])
+
+    def patch_defaults(self):
+        for (fct, idx, ft) in self._def_functions:
+            method, mod_name = self._fake_module_functions[
+                ft.__name__][ft.__module__]
+            fake_module = self.fake_modules[mod_name]
+            attr = method.__get__(fake_module, fake_module.__class__)
+            new_defaults = []
+            for i, d in enumerate(fct.__defaults__):
+                if i == idx:
+                    new_defaults.append(attr)
+                else:
+                    new_defaults.append(d)
+            fct.__defaults__ = tuple(new_defaults)
+
     def replace_globs(self, globs_):
         globs = globs_.copy()
         if self._isStale:
